package edu.uoc.tfg.yourlovelypet.repository;

import android.util.Log;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.EventListener;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.FirebaseFirestoreException;
import com.google.firebase.firestore.ListenerRegistration;
import com.google.firebase.firestore.QuerySnapshot;
import com.google.firebase.firestore.WriteBatch;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.model.Post;
import edu.uoc.tfg.yourlovelypet.model.Product;
import edu.uoc.tfg.yourlovelypet.model.User;

public class FirestoreRepository {
    private static final String TAG = "[FireStoreRepository]";
    private static final String COLLECTION_USERS = "users";
    private static final String COLLECTION_USER_PETS = "userPets";
    private static final String COLLECTION_USER_PRODUCTS = "userProducts";
    private static final String COLLECTION_GALLERY = "gallery";
    private static final String COLLECTION_POSTS = "posts";
    private static FirestoreRepository instance;
    private final FirebaseFirestore firebaseFirestore;
    private List<ListenerRegistration> listenerRegistrations = new ArrayList<>(); // to handle listeners at logout and avoid errors

    public FirestoreRepository() {
        firebaseFirestore = FirebaseFirestore.getInstance();
    }
    public static synchronized FirestoreRepository getInstance() {
        // Singleton pattern. Only one instance of FirebaseFirestore can be created
        if (instance == null) {
            instance = new FirestoreRepository();
        }
        return instance;
    }
    public interface Callback<T> {
        void onSuccess(T result);
        void onFailure(String error);
    }
    private void addListener(ListenerRegistration listenerRegistration) {
        listenerRegistrations.add(listenerRegistration);
    }
    public void clearListeners() {
        // To stop all listener at logout from MainViewModel
        for (ListenerRegistration registration : listenerRegistrations) {
            registration.remove();
            Log.d(TAG,"Removed listener: " + registration);
        }
        listenerRegistrations.clear();
    }
    public void addUser(User user, Callback<User> callback) {
        firebaseFirestore.collection(COLLECTION_USERS).document(user.getId()).set(user)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG,"New user data saved with id: " + user.getId());
                        callback.onSuccess(user);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.d(TAG,"Error saving user data with id: " + user.getId());
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void addPost(Post post, Callback<Post> callback) {
        firebaseFirestore.collection(COLLECTION_POSTS).add(post)
                .addOnSuccessListener(new OnSuccessListener<DocumentReference>() {
                    @Override
                    public void onSuccess(DocumentReference documentReference) {
                        String id = documentReference.getId();
                        documentReference.update("id", id)
                                .addOnSuccessListener(new OnSuccessListener<Void>() {
                                    @Override
                                    public void onSuccess(Void unused) {
                                        Log.d(TAG, "New post data saved at posts collection with id: " + id);
                                        callback.onSuccess(post);
                                    }
                                })
                                .addOnFailureListener(new OnFailureListener() {
                                    @Override
                                    public void onFailure(@NonNull Exception e) {
                                        Log.e(TAG, "Error updating assigned postId field at post with id: " + id, e);
                                        callback.onFailure(e.getMessage());
                                    }
                                });
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.d(TAG, "Error saving new post data at collection posts", e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void addPet(Pet pet, String userId, Callback<Pet> callback) {
        firebaseFirestore.collection(COLLECTION_USERS).document(userId).collection(COLLECTION_USER_PETS).add(pet)
                .addOnSuccessListener(new OnSuccessListener<DocumentReference>() {
                    @Override
                    public void onSuccess(DocumentReference documentReference) {
                        String id = documentReference.getId();
                        documentReference.update("id", id)
                                .addOnSuccessListener(new OnSuccessListener<Void>() {
                                    @Override
                                    public void onSuccess(Void unused) {
                                        Log.d(TAG, "New pet saved at userPets collection with id: " + id);
                                        pet.setId(id);
                                        callback.onSuccess(pet);
                                    }
                                })
                                .addOnFailureListener(new OnFailureListener() {
                                    @Override
                                    public void onFailure(@NonNull Exception e) {
                                        Log.e(TAG, "Error updating assigned petId field at pet with id: " + id, e);
                                        callback.onFailure(e.getMessage());
                                    }
                                });
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error saving new pet data at userPets", e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void addProduct(Product product, String userId, Callback<Product> callback) {
        firebaseFirestore.collection(COLLECTION_USERS).document(userId).collection(COLLECTION_USER_PRODUCTS).add(product)
                .addOnSuccessListener(new OnSuccessListener<DocumentReference>() {
                    @Override
                    public void onSuccess(DocumentReference documentReference) {
                        String id = documentReference.getId();
                        documentReference.update("id", id)
                                .addOnSuccessListener(new OnSuccessListener<Void>() {
                                    @Override
                                    public void onSuccess(Void unused) {
                                        Log.d(TAG, "New product saved at userProducts collection with id: " + id);
                                        product.setId(id);
                                        callback.onSuccess(product);
                                    }
                                })
                                .addOnFailureListener(new OnFailureListener() {
                                    @Override
                                    public void onFailure(@NonNull Exception e) {
                                        Log.e(TAG, "Error updating assigned productId field at product with id: " + id, e);
                                        callback.onFailure(e.getMessage());
                                    }
                                });
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error saving new product data at userProducts", e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void addNewImage(String userId, String petId, String fileName, Callback<Void> callback) {

        Map<String,String> map = new HashMap<>();
        map.put("filename", fileName);

        firebaseFirestore.collection(COLLECTION_USERS).document(userId)
                .collection(COLLECTION_USER_PETS).document(petId)
                .collection(COLLECTION_GALLERY).document(fileName).set(map)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG, "New pet image reference save with id (currentTimeMillis): " + fileName);
                        Log.d(TAG, "From pet with id: " + petId + " from user with id: " + userId);
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error saving new pet image reference with filename: " + fileName, e);
                        Log.d(TAG, "From pet with id: " + petId + " from user with id: " + userId);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void deletePet(String userId, String petId, Callback<Void> callback) {

        // Get pet document reference
        DocumentReference petReference = firebaseFirestore.collection(COLLECTION_USERS).document(userId)
                .collection(COLLECTION_USER_PETS).document(petId);

        // Use batch to add "delete pet" and "delete sub-collection gallery" operations
        WriteBatch batch = firebaseFirestore.batch();

        // Add "delete pet" operation to batch
        batch.delete(petReference);

        // Get sub-collection gallery reference
        CollectionReference galleryReference = petReference.collection("gallery");

        // Get all documents from sub-collection gallery
        galleryReference.get().addOnCompleteListener(new OnCompleteListener<QuerySnapshot>() {
            @Override
            public void onComplete(@NonNull Task<QuerySnapshot> task) {
                if (task.isSuccessful()) {
                    for (DocumentSnapshot document : task.getResult()) {
                        // Add "delete document from sub-collection gallery" to batch
                        batch.delete(document.getReference());
                    }

                    // Commit batch
                    batch.commit().addOnSuccessListener(new OnSuccessListener<Void>() {
                        @Override
                        public void onSuccess(Void unused) {
                            Log.d(TAG, "Pet with id: " + petId + " and its gallery have been deleted from user with id: " + userId);
                            //

                            callback.onSuccess(unused);
                        }
                    }).addOnFailureListener(new OnFailureListener() {
                        @Override
                        public void onFailure(@NonNull Exception e) {
                            Log.e(TAG, "Error deleting pet with id: " + petId + " and its gallery from user with id: " + userId, e);
                            callback.onFailure(e.getMessage());
                        }
                    });
                } else {
                    Log.e(TAG, "Error getting gallery documents for pet with id: " + petId, task.getException());
                    callback.onFailure(Objects.requireNonNull(task.getException()).getMessage());
                }
            }
        });
    }
    public void deleteImage(String userId, String petId, String filename, Callback<Void> callback) {
        firebaseFirestore.collection(COLLECTION_USERS).document(userId)
                .collection(COLLECTION_USER_PETS).document(petId)
                .collection(COLLECTION_GALLERY).document(filename).delete()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG, "Image with id: " + filename + " has been deleted from pet with id: " + petId + " from user with id: " + userId);
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error deleting image with id: " + filename + " from pet with id: " + petId + " from user with id: " + userId, e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void deleteProduct(String userId, String productId, Callback<Void> callback) {
        firebaseFirestore.collection(COLLECTION_USERS).document(userId)
                .collection(COLLECTION_USER_PRODUCTS).document(productId).delete()
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG, "Product with id: " + productId + " has been deleted from user with id: " + userId);
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error deleting product with id: " + productId + " from user with id: " + userId, e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void updateUserLanguage(User userToUpdate, Callback<Void> callback) {
        firebaseFirestore.collection(COLLECTION_USERS).document(userToUpdate.getId()).update("userLanguage", userToUpdate.getUserLanguage())
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG, "User language updated");
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error updating user language", e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void updateUser(User userToUpdate, Callback<Void> callback) {
        String userId = userToUpdate.getId();
        firebaseFirestore.collection(COLLECTION_USERS).document(userId).set(userToUpdate)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG, "User with id " + userId + " has been updated");
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error updating pet date with id: " + userId + " from user with id: " + userId, e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void updatePet(Pet petToUpdate, String userId, Callback<Void> callback) {
        String petId = petToUpdate.getId();
        firebaseFirestore.collection(COLLECTION_USERS)
                .document(userId).collection(COLLECTION_USER_PETS)
                .document(petId).set(petToUpdate)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG, "Pet with id " + petId + " has been updated");
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error updating pet date with id: " + petId + " from user with id: " + userId, e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void updateProfileImage(String userId, String petId, String filename, Callback<Void> callback) {
        firebaseFirestore.collection(COLLECTION_USERS)
                .document(userId).collection(COLLECTION_USER_PETS)
                .document(petId).update("imageProfile", filename)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
                    @Override
                    public void onSuccess(Void unused) {
                        Log.d(TAG,"Image profile updated of pet: " + petId + " to image: " + filename);
                        callback.onSuccess(unused);
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG,"Error updating profile image of pet: " + petId + " : ", e);
                        callback.onFailure(e.getMessage());
                    }
                });
    }
    public void getUser(String userId, Callback<User> callback) {
        ListenerRegistration listenerRegistration =
                firebaseFirestore.collection(COLLECTION_USERS).document(userId)
                        .addSnapshotListener(new EventListener<DocumentSnapshot>() {
                            @Override
                            public void onEvent(@Nullable DocumentSnapshot value, @Nullable FirebaseFirestoreException e) {
                                if (e != null) {
                                    Log.e(TAG, "Error getting current user data: ", e);
                                    callback.onFailure(e.getMessage());
                                    return;
                                }

                                User user = Objects.requireNonNull(value).toObject(User.class);
                                callback.onSuccess(user);
                            }
                        });
        addListener(listenerRegistration);
    }
    public void getPetsFromUser(String userId, Callback<List<Pet>> callback) {
        // SnapshotListener of "sub-collection" pets of document user
        ListenerRegistration listenerRegistration =
                firebaseFirestore.collection(COLLECTION_USERS).document(userId).collection(COLLECTION_USER_PETS)
                .addSnapshotListener(new EventListener<QuerySnapshot>() {
                    @Override
                    public void onEvent(@Nullable QuerySnapshot queryDocumentSnapshots, @Nullable FirebaseFirestoreException e) {
                        if (e != null) {
                            Log.e(TAG, "Error getting user pets data: ", e);
                            callback.onFailure(e.getMessage());
                            return;
                        }

                        List<Pet> pets = new ArrayList<>();
                        for (DocumentSnapshot snapshot : Objects.requireNonNull(queryDocumentSnapshots)) {
                            Pet pet = snapshot.toObject(Pet.class);
                            if (pet != null) {
                                pet.setId(snapshot.getId());
                                pets.add(pet);
                            }
                        }
                        callback.onSuccess(pets);
                    }
                });
        addListener(listenerRegistration);
    }
    public void getProductsFromUser(String userId, Callback<List<Product>> callback) {
        // SnapshotListener of "sub-collection" pets of document user
        ListenerRegistration listenerRegistration =
                firebaseFirestore.collection(COLLECTION_USERS).document(userId).collection(COLLECTION_USER_PRODUCTS)
                        .addSnapshotListener(new EventListener<QuerySnapshot>() {
                            @Override
                            public void onEvent(@Nullable QuerySnapshot queryDocumentSnapshots, @Nullable FirebaseFirestoreException e) {
                                if (e != null) {
                                    Log.e(TAG, "Error getting user products data: ", e);
                                    callback.onFailure(e.getMessage());
                                    return;
                                }

                                List<Product> products = new ArrayList<>();
                                for (DocumentSnapshot snapshot : Objects.requireNonNull(queryDocumentSnapshots)) {
                                    Product product = snapshot.toObject(Product.class);
                                    if (product != null) {
                                        product.setId(snapshot.getId());
                                        products.add(product);
                                    }
                                }
                                callback.onSuccess(products);
                            }
                        });
        addListener(listenerRegistration);
    }
    public void getPosts(Callback<List<Post>> callback) {
        // SnapshotListener of collection posts
        ListenerRegistration listenerRegistration =
                firebaseFirestore.collection(COLLECTION_POSTS)
                .addSnapshotListener(new EventListener<QuerySnapshot>() {
                    @Override
                    public void onEvent(@Nullable QuerySnapshot queryDocumentSnapshots, @Nullable FirebaseFirestoreException e) {
                        if (e != null) {
                            Log.e(TAG, "Error getting posts data: ", e);
                            callback.onFailure(e.getMessage());
                            return;
                        }
                        if (queryDocumentSnapshots != null) {
                            List<Post> posts = new ArrayList<>();
                            for (DocumentSnapshot snapshot : queryDocumentSnapshots) {
                                Post post = snapshot.toObject(Post.class);
                                if (post != null) {
                                    post.setId(snapshot.getId());
                                    posts.add(post);
                                }
                            }
                            callback.onSuccess(posts);
                        } else {
                            callback.onFailure("QuerySnapshot is null");
                        }
                    }
                });
        addListener(listenerRegistration);
    }
    public LiveData<List<String>> getGalleryFromUserPet(String userId, String petId) {
        MutableLiveData<List<String>> data = new MutableLiveData<>();
        // SnapshotListener of "sub-collection" gallery of pet
        ListenerRegistration listenerRegistration =
                firebaseFirestore.collection(COLLECTION_USERS).document(userId).collection(COLLECTION_USER_PETS).document(petId).collection(COLLECTION_GALLERY)
                .addSnapshotListener(new EventListener<QuerySnapshot>() {
                    @Override
                    public void onEvent(@Nullable QuerySnapshot queryDocumentSnapshots, @Nullable FirebaseFirestoreException e) {
                        if (e != null) {
                            Log.e(TAG, "Error getting gallery data from pet with id: " + petId + "from user with id: " + userId , e);
                            return;
                        }
                        if ( (queryDocumentSnapshots != null) && !queryDocumentSnapshots.isEmpty()) {
                            List<String> gallery = new ArrayList<>();
                            for (DocumentSnapshot snapshot : queryDocumentSnapshots) {
                                String filename = snapshot.getString("filename");
                                String fileReference = userId + "/" + petId + "/" + filename;
                                gallery.add(fileReference);
                            }
                            data.setValue(gallery);
                        } else {
                            Log.e(TAG, "Gallery is empty for pet with id: " + petId + "from user with id: " + userId , e);
                            data.setValue(new ArrayList<>());
                        }
                    }
                });
        addListener(listenerRegistration);
        return data;
    }
    public LiveData<String> getCurrentUserLanguage(String userId) {
        MutableLiveData<String> userLanguage = new MutableLiveData<>();
        firebaseFirestore.collection(COLLECTION_USERS).document(userId).get()
                .addOnSuccessListener(new OnSuccessListener<DocumentSnapshot>() {
                    @Override
                    public void onSuccess(DocumentSnapshot documentSnapshot) {
                        if (documentSnapshot.exists()) {
                            String userLanguageDb = documentSnapshot.getString("userLanguage");
                            Log.d(TAG,"User preferred language read from db is: " + userLanguageDb);
                            userLanguage.setValue(userLanguageDb);
                        }
                    }
                })
                .addOnFailureListener(new OnFailureListener() {
                    @Override
                    public void onFailure(@NonNull Exception e) {
                        Log.e(TAG, "Error getting user with id: " + userId + " preferred language: ", e);
                        userLanguage.setValue("en"); // Default value in error case
                    }
                });
        return userLanguage;
    }
}
