package edu.uoc.tfg.yourlovelypet.views;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.util.Patterns;
import android.view.View;
import android.widget.Toast;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.ActivityLoginBinding;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.utils.Result;
import edu.uoc.tfg.yourlovelypet.viewModel.LoginViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.LoginWelcomeDialogFragment;

public class LoginActivity extends AppCompatActivity {

    private static final String TAG = "[LoginActivity]";
    private static final int SPLASH_DIALOG = 3000;
    private ActivityLoginBinding binding;
    private LoginViewModel loginViewModel;
    private LoginWelcomeDialogFragment welcomeAgain;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityLoginBinding.inflate(getLayoutInflater());
        View view = binding.getRoot();
        setContentView(view);
        loginViewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        // verify if user is already logged
        String userId = getUserIdFromPreferences();
        if (userId != null) {
            navigateToMainActivity(userId);
        } else {
            setupObservers();
            initUI();
        }
    }

    private void setupObservers() {

        // Observer login operation success LiveData
        loginViewModel.getLoginResult().observe(this, new Observer<Result>() {
            public void onChanged(Result loginResult) {
                if (loginResult.isSuccess()) {
                    String userId = loginResult.getUserId();
                    Log.d(TAG, "User with id: " + userId + " has just logged in");

                    loginViewModel.getUser().observe(LoginActivity.this, new Observer<User>() {
                        @Override
                        public void onChanged(User user) {
                            if (user != null) {
                                // User welcome with his name
                                String name = user.getUserName();
                                welcomeAgain = LoginWelcomeDialogFragment.newInstance(name);
                                welcomeAgain.show(getSupportFragmentManager(), "welcomeAgain");

                                new Handler().postDelayed(new Runnable() {
                                    @Override
                                    public void run() {
                                        // Remove LoginWelcomeDialogFragment if exists
                                        Fragment fm = getSupportFragmentManager().findFragmentByTag("welcomeAgain");
                                        if (fm != null) getSupportFragmentManager().beginTransaction().remove(fm).commit();
                                        navigateToMainActivity(userId);
                                    }
                                }, SPLASH_DIALOG);
                            }
                        }
                    });
                } else {
                    Log.d(TAG,"Login process failed: " + loginResult.getError());
                    Toast.makeText(getApplicationContext(), getString(R.string.login_login_failed_try_again), Toast.LENGTH_SHORT).show();
                }
            }
        });

    }

    private void initUI() {

        binding.buttonLogin.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String email = binding.editEmail.getEditableText().toString();
                String password = binding.editPassword.getEditableText().toString();

                if (validateFields(email,password)) {
                    loginViewModel.login(email, password);
                }
            }
        });

        binding.tvSignUp.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent signupIntent = new Intent(getApplicationContext(), SignupActivity.class);
                startActivity(signupIntent);
                finish();
            }
        });
    }
    private boolean validateFields(String email, String password) {

        if (TextUtils.isEmpty(email)) {
            binding.editEmail.setError(getString(R.string.login_error_email_required));
            return false;
        }

        if (!Patterns.EMAIL_ADDRESS.matcher(email).matches()) {
            binding.editEmail.setError(getString(R.string.login_error_email_invalid));
            return false;
        }

        if (TextUtils.isEmpty(password)) {
            binding.editPassword.setError(getString(R.string.login_error_password_required));
            return false;
        }

        return true;
    }

    private String getUserIdFromPreferences() {
        SharedPreferences sharedPreferences = getSharedPreferences("MyAppPrefs", Context.MODE_PRIVATE);
        return sharedPreferences.getString("userId", null);
    }

    private void navigateToMainActivity(String userId) {
        Intent intent = new Intent(getApplicationContext(), MainActivity.class);
        intent.putExtra("userId", userId);
        startActivity(intent);
        finish();
    }
}