package edu.uoc.tfg.yourlovelypet.views;

import android.content.Intent;
import android.os.Bundle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import android.os.Handler;
import android.text.TextUtils;
import android.util.Log;
import android.util.Patterns;
import android.view.View;
import android.widget.Toast;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.ActivitySignupBinding;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.utils.Result;
import edu.uoc.tfg.yourlovelypet.viewModel.SignupViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.SignupWelcomeDialogFragment;

public class SignupActivity extends AppCompatActivity {

    private static final String TAG = "[SignupActivity]";
    private static final int SPLASH_DIALOG = 3000;
    private ActivitySignupBinding binding;
    private SignupViewModel signupViewModel;
    private SignupWelcomeDialogFragment signupWelcomeDialogFragment;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivitySignupBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        signupViewModel = new ViewModelProvider(this).get(SignupViewModel.class);
        setupObservers();
        initUI();
    }

    private void setupObservers() {

        // Observer signup operation success LiveData
        signupViewModel.getSignupResult().observe(this, new Observer<Result>() {
            public void onChanged(Result signupResult) {
                if (signupResult.isSuccess()) {
                    String userId = signupResult.getUserId();
                    Log.d(TAG, "User with id: " + userId + " has just signup");

                    signupViewModel.getUser().observe(SignupActivity.this, new Observer<User>() {
                        @Override
                        public void onChanged(User user) {
                            if (user != null) {
                                // New user welcome with his name
                                String name = user.getUserName();
                                signupWelcomeDialogFragment = SignupWelcomeDialogFragment.newInstance(name);
                                signupWelcomeDialogFragment.show(getSupportFragmentManager(), "welcomeSignup");

                                new Handler().postDelayed(new Runnable() {
                                    @Override
                                    public void run() {
                                        // Remove signupWelcomeDialogFragment if exists
                                        Fragment fm = getSupportFragmentManager().findFragmentByTag("welcomeSignup");
                                        if (fm != null) getSupportFragmentManager().beginTransaction().remove(fm).commit();
                                        // Go to MainActivity sending userId
                                        Intent intent = new Intent(getApplicationContext(), MainActivity.class);
                                        intent.putExtra("userId", userId);
                                        startActivity(intent);
                                        finish();
                                    }
                                }, SPLASH_DIALOG);
                            }
                        }
                    });
                } else {
                    Log.d(TAG, "Sign up process failed: " + signupResult.getError());
                    Toast.makeText(getApplicationContext(), getString(R.string.signup_signup_failed), Toast.LENGTH_SHORT).show();
                }
            }
        });
    }

    private void initUI() {
        binding.button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String name = binding.editUsername.getEditableText().toString();
                String email = binding.editEmail.getEditableText().toString();
                String password = binding.editPassword.getEditableText().toString();
                String repeatPassword = binding.editRepeatPassword.getEditableText().toString();
                String language = "es";
                if (binding.language.getCheckedRadioButtonId() == R.id.english) language = "en";

                if (validateFields(name, email, password, repeatPassword)) {
                    signupViewModel.signup(name, email, password, language);
                }
            }
        });
    }

    private boolean validateFields(String username, String email, String password, String repeatPassword) {

        if (TextUtils.isEmpty(username)) {
            binding.editUsername.setError(getString(R.string.signup_error_username_required));
            return false;
        }

        if (TextUtils.isEmpty(email)) {
            binding.editEmail.setError(getString(R.string.signup_error_email_required));
            return false;
        }

        if (!Patterns.EMAIL_ADDRESS.matcher(email).matches()) {
            binding.editEmail.setError(getString(R.string.signup_error_email_invalid));
            return false;
        }

        if (TextUtils.isEmpty(password)) {
            binding.editPassword.setError(getString(R.string.signup_error_password_required));
            return false;
        }

        if (password.length() < 6) {
            binding.editPassword.setError(getString(R.string.signup_error_password_length));
            return false;
        }

        if (!password.equals(repeatPassword)) {
            binding.editRepeatPassword.setError(getString(R.string.signup_error_password_mismatch));
            return false;
        }

        return true;
    }
}