package edu.uoc.tfg.yourlovelypet.views.adapters;

import android.content.Context;
import android.graphics.Color;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.TextView;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.PopupMenu;
import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.ProductRowBinding;
import edu.uoc.tfg.yourlovelypet.model.Product;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.dialogs.ConfirmProductDeleteDialogFragment;

public class ProductAdapter extends RecyclerView.Adapter<ProductAdapter.ViewHolder> {

    private static final String TAG = "[ProductAdapter]";
    private MainViewModel mainViewModel;
    private Context context;
    private List<Product> dataSet = new ArrayList<>(); // List of products to send

    public ProductAdapter(MainViewModel mainViewModel, Context context) {
        this.mainViewModel = mainViewModel;
        this.context = context;
    }

    public class ViewHolder extends RecyclerView.ViewHolder implements View.OnClickListener, PopupMenu.OnMenuItemClickListener {

        private ProductRowBinding recyclerRowBinding;
        private final TextView tvPetName;
        private final TextView tvProductName;
        private final TextView tvProductDaily;
        private final TextView tvProductUnits;
        private final TextView tvDaysOff;
        private final ImageButton menu;
        private PopupMenu popupMenu;

        public ViewHolder(ProductRowBinding recyclerRowBinding) {
            super(recyclerRowBinding.getRoot());
            this.recyclerRowBinding = recyclerRowBinding;

            tvPetName = recyclerRowBinding.tvPetName;
            tvProductName = recyclerRowBinding.tvProductName;
            tvProductDaily = recyclerRowBinding.tvProductDaily;
            tvProductUnits = recyclerRowBinding.tvProductUnits;
            tvDaysOff = recyclerRowBinding.tvDaysOff;
            menu = recyclerRowBinding.tvProductMenu;
            menu.setOnClickListener(this);
        }

        public TextView getTvPetName() {
            return tvPetName;
        }
        public TextView getTvProductName() {
            return tvProductName;
        }
        public TextView getTvProductDaily() {
            return tvProductDaily;
        }
        public TextView getTvProductUnits() {
            return tvProductUnits;
        }
        public TextView getTvDaysOff() {
            return tvDaysOff;
        }

        @Override
        public void onClick(View view) {
            showMenu(view);
        }

        private void showMenu(View view) {
            popupMenu = new PopupMenu(view.getContext(), view);
            popupMenu.inflate(R.menu.product_menu);
            popupMenu.setOnMenuItemClickListener(this);
            popupMenu.show();
        }

        @Override
        public boolean onMenuItemClick(MenuItem item) {
            int position = getBindingAdapterPosition();

            if (position != RecyclerView.NO_POSITION) {
                Product productToEdit = dataSet.get(position);
                String productId = productToEdit.getId();

                if (item.getItemId() == R.id.edit) {
                    Log.d(TAG, "Clicked option edit at menu over product with Id: " + productId + " its name is: " + productToEdit.getName());
                    // TODO at future developments
                    return true;
                } else if (item.getItemId() == R.id.delete) {
                    Log.d(TAG, "Clicked option delete at menu over product with Id: " + productId + " its name is: " + productToEdit.getName());
                    ConfirmProductDeleteDialogFragment confirmProductDeleteDialogFragment = ConfirmProductDeleteDialogFragment.newInstance(productId);
                    confirmProductDeleteDialogFragment.show(((AppCompatActivity) context).getSupportFragmentManager(), "confirmDeleteProduct");
                    return true;
                } else if (item.getItemId() == R.id.addUnits) {
                    Log.d(TAG, "Clicked option addUnits at menu over product with Id: " + productId + " its name is: " + productToEdit.getName());
                    // TODO at future developments
                    return true;
                } else {
                    return false;
                }
            }
            return false;
        }
    }

    public void setData(List<Product> products) {
        this.dataSet = products;
        this.notifyDataSetChanged(); // ViewModel: notify to observers
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int viewType) {
        LayoutInflater layoutInflater = LayoutInflater.from(viewGroup.getContext());
        ProductRowBinding recyclerRowBinding = ProductRowBinding.inflate(layoutInflater, viewGroup, false);
        return new ViewHolder(recyclerRowBinding);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder viewHolder, final int position) {
        Product product = dataSet.get(position);
        viewHolder.getTvPetName().setText(mainViewModel.getPetById(product.getPetId()).getPetName());
        viewHolder.getTvProductName().setText(product.getName());
        viewHolder.getTvProductDaily().setText(product.getDaily());
        viewHolder.getTvProductUnits().setText(product.getUnits());
        
        int daysOff = calculateDaysOff(product);
        String daysOffMessage = daysOff + " " + context.getString(R.string.days);
        viewHolder.getTvDaysOff().setText(daysOffMessage);


        if (daysOff < 3) {
            viewHolder.getTvDaysOff().setTextColor(Color.RED);
        } else if (daysOff < 5) {
            viewHolder.getTvDaysOff().setTextColor(Color.MAGENTA);
        } else {
            viewHolder.getTvDaysOff().setTextColor(ContextCompat.getColor(context, R.color.darkBlue));
        }
    }

    @Override
    public int getItemCount() {
        return dataSet.size();
    }

    private int calculateDaysOff(Product product) {

        // Get initDate from product
        SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
        Calendar initCalendar = Calendar.getInstance();
        try {
            initCalendar.setTime(dateFormat.parse(product.getInitDate()));
        } catch (ParseException e) {
            Log.e(TAG, "Error parsing initDate: " + e.getMessage());
            return 0;
        }

        // Get current day and calculate days since init
        Calendar today = Calendar.getInstance();
        long diff = today.getTimeInMillis() - initCalendar.getTimeInMillis();
        int daysSinceInit = (int) (diff / (1000 * 60 * 60 * 24));

        // Calculate remaining quantity
        double dailyConsumption = Double.parseDouble(product.getDaily());
        double initialQuantity = Double.parseDouble(product.getInitQuantity());
        double remainingQuantity = initialQuantity - (daysSinceInit * dailyConsumption);

        if (remainingQuantity <= 0) {
            return 0; // In case of negative values return 0
        }

        // Rounding down
        return (int) Math.floor(remainingQuantity / dailyConsumption);
    }
}
