package edu.uoc.tfg.yourlovelypet.views.dialogs;

import android.app.Dialog;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AutoCompleteTextView;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.lifecycle.ViewModelProvider;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentAddPetProductDialogBinding;
import edu.uoc.tfg.yourlovelypet.model.Product;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;

public class AddPetProductDialogFragment extends DialogFragment {
    private static final String TAG = "[AddPetProduct]";
    private static final String ARG_PET_ID = "petId";
    private FragmentAddPetProductDialogBinding binding;
    private MainViewModel mainViewModel;
    private AutoCompleteTextView autoCompleteProductUnits;
    private String petId;

    public AddPetProductDialogFragment() {}

    public static AddPetProductDialogFragment newInstance(String param) {
        AddPetProductDialogFragment fragment = new AddPetProductDialogFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PET_ID, param);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            petId = getArguments().getString(ARG_PET_ID);
            Log.d(TAG,"Receive petId: " + petId);
        }
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(@Nullable Bundle savedInstanceState) {
        Dialog dialog = super.onCreateDialog(savedInstanceState);
        if (dialog.getWindow() != null) {
            dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            dialog.getWindow().setBackgroundDrawableResource(R.drawable.custom_bg_heaven);
        }
        return dialog;
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentAddPetProductDialogBinding.inflate(getLayoutInflater(), container, false);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        autoCompleteProductUnits = binding.editUnits;
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        binding.editUnits.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                autoCompleteProductUnits.showDropDown();
            }
        });

        binding.buttonAdd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if (petId == null) {
                    Log.e(TAG, "Error petId not received. Can't add a new product");
                    return;
                }

                String name = binding.editName.getEditableText().toString();
                String daily = binding.editDaily.getEditableText().toString();
                String units = binding.editUnits.getEditableText().toString();
                String initQuantity = binding.editInitQuantity.getEditableText().toString();
                // Convert to english in case of units spanish input
                String englishUnits = convertToEnglish(units);

                if (validateFields(name, daily, englishUnits, initQuantity)) {

                    // Get current date
                    Calendar calendar = Calendar.getInstance();
                    SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault());
                    String currentDate = dateFormat.format(calendar.getTime());

                    Product newProduct = new Product(petId, name, englishUnits, daily, initQuantity, currentDate);
                    Log.d(TAG, "Request to add a new product with name: " + name);
                    mainViewModel.addProduct(newProduct);
                    dismiss();
                }
            }
        });

        binding.buttonCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });
    }

    private boolean validateFields(String name, String daily, String units, String initQuantity) {

        if (TextUtils.isEmpty(name)) {
            binding.editName.setError(getString(R.string.addProduct_name_required));
            return false;
        }

        if (TextUtils.isEmpty(daily)) {
            binding.editDaily.setError(getString(R.string.addProduct_daily_required));
            return false;
        } else {
            try {
                Double.parseDouble(daily);
            } catch (NumberFormatException e) {
                binding.editDaily.setError(getString(R.string.addProduct_daily_number));
                return false;
            }
        }

        if (TextUtils.isEmpty(initQuantity)) {
            binding.editInitQuantity.setError(getString(R.string.addProduct_init_quantity_required));
            return false;
        } else {
            try {
                Double.parseDouble(initQuantity);
            } catch (NumberFormatException e) {
                binding.editInitQuantity.setError(getString(R.string.addProduct_init_quantity_number));
                return false;
            }
        }

        if (TextUtils.isEmpty(units)) {
            binding.editUnits.setError(getString(R.string.addProduct_units));
            return false;
        }

        return true;
    }

    private String convertToEnglish(String bilingualUnits) {
        switch (bilingualUnits) {
            case "gramos":
            case "grams":
                return "grams";
            default:
                return "units";
        }
    }
}
