package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.Manifest;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import edu.uoc.tfg.yourlovelypet.model.Pet;
import edu.uoc.tfg.yourlovelypet.repository.FirestorageRepository;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import edu.uoc.tfg.yourlovelypet.views.adapters.PhotoAdapter;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentGalleryBinding;

/*
	- El observer de GalleryFragment observa una lista de urls guardada/devuelta como List<String> por método getGalleryFromUserPet
	  Al cambiar, de cada elemento de la lista obtiene el StorageReference (por lo que necesita el path/fileName con que se guarde) y pide la url.
	  Cuando las tiene todas las pasa a photoAdapeter, donde usando Glide cargará las imágenes.

	- El método MainViewModel.getGalleryFromUserPet(petId):
			Devuelve el return del FireStoreRepository.getGalleryFromUserPet(userId, petId)

	- El método FireStoreRepository.getGalleryFromUserPet(userId, petId):
			Devuelve un LiveData de List<String> dónde cada elemento es construido a partir del valor en la subcolección gallery (donde tenemos/debemos
			registrar el nombre del archivo, filename) añadiendole el path correspondiente al user y a la pet.

	- Al añadir una nueva imagen a la gallery (en firestore) solamente debemos registrar un nombre para identificar la imagen (System.currentTimeMillis() + ".jpg").
	  Operación que realizamos en el método upload de GalleryFragment llamando a MainViewModel que a su vez llama a FirestoreRepository, donde se añade el nuevo
	  document (con id = valor de filename y un campo llamado filename cuyo valor es el valor de filename también).
			nota: para obtener la StorageReference donde subir el archivo si necesitamos path/filename
			nota: imageUri hace referencia al archivo obtenido del dispositivo
 */

public class GalleryFragment extends Fragment {

    private static final String TAG = "[GalleryFragment]";
    private FragmentGalleryBinding binding;
    private MainViewModel mainViewModel;
    private RecyclerView recyclerView;
    private static final String ARG_PET_ID = "petId";
    private String receivedPetId;
    private String userId = "";
    private Pet currentPet;
    private PhotoAdapter photoAdapter; // recyclerView
    private Uri imageUri;

    private final ActivityResultLauncher<String> getContentLauncher = registerForActivityResult(
            new ActivityResultContracts.GetContent(),
            uri -> {
                if (uri != null) {
                    imageUri = uri;
                    upload(imageUri);
                }
            }
    );

    private final ActivityResultLauncher<String> requestPermissionLauncher = registerForActivityResult(
            new ActivityResultContracts.RequestPermission(),
            isGranted -> {
                if (isGranted) {
                    openImageChooser();
                } else {
                    Toast.makeText(getContext(), "Permission denied", Toast.LENGTH_SHORT).show();
                }
            }
    );

    public GalleryFragment() {}

    public static GalleryFragment newInstance(String param) {
        GalleryFragment fragment = new GalleryFragment();
        Bundle args = new Bundle();
        args.putString(ARG_PET_ID, param);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            receivedPetId = getArguments().getString(ARG_PET_ID);
            Log.d(TAG,"Receive petId: " + receivedPetId + " from PetAdapter/HomeFragment");
        }
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentGalleryBinding.inflate(getLayoutInflater(), container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);
        currentPet = mainViewModel.getPetById(receivedPetId);
        userId = Objects.requireNonNull(mainViewModel.getCurrentUser().getValue()).getId();
        binding.textViewPetName.setText(currentPet.getPetName());

        // RecyclerView and Adapter
        recyclerView = binding.recyclerViewGallery;
        // Send context to show new dialogFragment from there and petId
        photoAdapter = new PhotoAdapter(receivedPetId, getContext());
        recyclerView.setAdapter(photoAdapter);
        recyclerView.setLayoutManager(new GridLayoutManager(getContext(),2));

        // Observer of gallery (list of images) of pet (MainViewModel) and send urls to PhotoAdapter
        mainViewModel.getGalleryFromUserPet(receivedPetId).observe(getViewLifecycleOwner(), new Observer<List<String>>() {
            @Override
            public void onChanged(List<String> galleryReferences) {
                List<String> urls = new ArrayList<>();
                for (String fileReference : galleryReferences) {
                    mainViewModel.getUrl(fileReference, new FirestorageRepository.Callback<Uri>() {
                        @Override
                        public void onSuccess(Uri uri) {
                            urls.add(uri.toString());
                            // To avoid async problems. Check if all urls have been added
                            if (urls.size() == galleryReferences.size()) {
                                photoAdapter.setData(urls);
                            }
                        }
                        @Override
                        public void onFailure(String error) {
                            Log.e(TAG,"Error getting url: " + error);
                        }
                    });
                }
            }
        });

        // Observe addImage operation success LiveData
        mainViewModel.getAddImageSuccess().observe(getViewLifecycleOwner(), new Observer<Boolean>() {
            @Override
            public void onChanged(Boolean success) {
                if (success != null && success) {
                    Log.d(TAG, "Image added successfully");
                    mainViewModel.resetAddImageSuccess(); // Reset Livedata value
                    Toast.makeText(getContext(), "Image added", Toast.LENGTH_SHORT).show(); // TODO replace with string resource
                }
            }
        });

        // Observe addImage operation error LiveData
        mainViewModel.getAddImageError().observe(getViewLifecycleOwner(), new Observer<String>() {
            @Override
            public void onChanged(String error) {
                if (error != null) {
                    Log.d(TAG, "Error adding image: " + error);
                    mainViewModel.resetAddImageError(); // Reset Livedata value
                    Toast.makeText(getContext(), "Error adding image, please try again", Toast.LENGTH_SHORT).show(); // TODO replace with string resource
                }
            }
        });

        binding.addImageButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.READ_MEDIA_IMAGES) == PackageManager.PERMISSION_GRANTED) {
                    openImageChooser();

                } else {
                    requestPermissionLauncher.launch(Manifest.permission.READ_MEDIA_IMAGES);
                }
            }
        });
    }

    private void openImageChooser() {
        getContentLauncher.launch("image/*");
    }

    private void upload(Uri imageUri) {
        if (imageUri != null) {
            String path = userId + "/" + receivedPetId + "/" ;
            String fileName = System.currentTimeMillis() + ".jpg";
            Log.d(TAG,"New image to upload at: " + path + fileName);
            mainViewModel.uploadImage(imageUri, receivedPetId, path, fileName);
        } else {
            Toast.makeText(getContext(), "No file selected", Toast.LENGTH_SHORT).show();
            Log.d(TAG,"No file selected");
        }
    }
}