package edu.uoc.tfg.yourlovelypet.views.fragments;

import android.Manifest;
import android.content.pm.PackageManager;
import android.location.Location;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.SearchView;
import android.widget.Toast;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import edu.uoc.tfg.yourlovelypet.R;
import edu.uoc.tfg.yourlovelypet.databinding.FragmentSearchBinding;
import edu.uoc.tfg.yourlovelypet.model.User;
import edu.uoc.tfg.yourlovelypet.viewModel.MainViewModel;
import com.google.android.libraries.places.api.Places;
import com.google.android.libraries.places.api.model.AutocompletePrediction;
import com.google.android.libraries.places.api.model.Place;
import com.google.android.libraries.places.api.model.RectangularBounds;
import com.google.android.libraries.places.api.net.FetchPlaceRequest;
import com.google.android.libraries.places.api.net.FetchPlaceResponse;
import com.google.android.libraries.places.api.net.FindAutocompletePredictionsRequest;
import com.google.android.libraries.places.api.net.FindAutocompletePredictionsResponse;
import com.google.android.libraries.places.api.net.PlacesClient;
import java.util.Arrays;
import java.util.concurrent.atomic.AtomicInteger;

public class SearchFragment extends Fragment {

    private static final String TAG = "[SearchFragment]";
    private FragmentSearchBinding binding;
    private MainViewModel mainViewModel;
    private GoogleMap map;
    private PlacesClient placesClient;
    private FusedLocationProviderClient fusedLocationClient;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Initialize Places
        Places.initialize(requireContext(), getString(R.string.google_maps_key));
        placesClient = Places.createClient(requireContext());

        // Initialize FusedLocationProviderClient
        fusedLocationClient = LocationServices.getFusedLocationProviderClient(requireContext());
    }

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        binding = FragmentSearchBinding.inflate(getLayoutInflater(), container, false);
        return binding.getRoot();
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        mainViewModel = new ViewModelProvider(requireActivity()).get(MainViewModel.class);

        // Observer of current user (MainViewModel)
        mainViewModel.getCurrentUser().observe(getViewLifecycleOwner(), new Observer<User>() {
            @Override
            public void onChanged(User user) {
                binding.textViewUserName.setText(user.getUserName());
            }
        });

        SupportMapFragment supportMapFragment = (SupportMapFragment) getChildFragmentManager().findFragmentById(R.id.map_fragment);

        if (supportMapFragment != null) {
            supportMapFragment.getMapAsync(new OnMapReadyCallback() {
                @Override
                public void onMapReady(@NonNull GoogleMap googleMap) {
                    map = googleMap;
                    checkPermission();
                }
            });
        }

        // Set up SearchView
        binding.mapSearch.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
                searchNearbyPlaces(query);
                return true;
            }

            @Override
            public boolean onQueryTextChange(String newText) {
                return false;
            }
        });
    }

    private void checkPermission() {

        // Check permission to access at user location
        if (ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            map.setMyLocationEnabled(true);
            fusedLocationClient.getLastLocation() // Get last known location
                    .addOnSuccessListener(requireActivity(), new OnSuccessListener<Location>() {
                        @Override
                        public void onSuccess(Location location) {
                            if (location != null) {
                                LatLng currentLocation = new LatLng(location.getLatitude(), location.getLongitude());
                                map.animateCamera(CameraUpdateFactory.newLatLngZoom(currentLocation, 15f), 4000, null); // Center map at location
                            }
                        }
                    })
                    .addOnFailureListener(requireActivity(), new OnFailureListener() {
                        @Override
                        public void onFailure(@NonNull Exception e) {
                            Log.e(TAG, "Error getting last known location: " + e.getMessage());
                        }
                    });
        } else {
            Toast.makeText(getActivity(), getString(R.string.search_permission), Toast.LENGTH_SHORT).show();
        }
    }

    private void searchNearbyPlaces(String query) {
        Log.d(TAG, "New query: " + query);

        // Check permission to access at user location
        if (ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            fusedLocationClient.getLastLocation() // Get last known location
                    .addOnSuccessListener(requireActivity(), new OnSuccessListener<Location>() {
                        @Override
                        public void onSuccess(Location location) {
                            if (location != null) {
                                LatLng currentLocation = new LatLng(location.getLatitude(), location.getLongitude());

                                // Perform request to get autocomplete predictions
                                FindAutocompletePredictionsRequest request = FindAutocompletePredictionsRequest.builder()
                                        .setLocationBias(RectangularBounds.newInstance(
                                                new LatLng(currentLocation.latitude - 0.1, currentLocation.longitude - 0.1),
                                                new LatLng(currentLocation.latitude + 0.1, currentLocation.longitude + 0.1)
                                        ))
                                        .setQuery(query)
                                        .build();

                                // Send request using Places API
                                placesClient.findAutocompletePredictions(request)
                                        .addOnSuccessListener(new OnSuccessListener<FindAutocompletePredictionsResponse>() {
                                            @Override
                                            public void onSuccess(FindAutocompletePredictionsResponse response) {
                                                Log.d(TAG,"Query response: " + response);
                                                map.clear();
                                                final LatLng[] nearestPlaceLatLng = {null};
                                                final double[] minDistance = {Double.MAX_VALUE};
                                                int totalPlaces = response.getAutocompletePredictions().size();
                                                AtomicInteger placesProcessed = new AtomicInteger(0);

                                                for (AutocompletePrediction prediction : response.getAutocompletePredictions()) {
                                                    String placeId = prediction.getPlaceId();

                                                    // Perform request to get more details about found place
                                                    FetchPlaceRequest placeRequest = FetchPlaceRequest.builder(placeId, Arrays.asList(Place.Field.ID, Place.Field.NAME, Place.Field.LAT_LNG)).build();

                                                    // Send request using Places API
                                                    placesClient.fetchPlace(placeRequest)
                                                            .addOnSuccessListener(new OnSuccessListener<FetchPlaceResponse>() {
                                                                @Override
                                                                public void onSuccess(FetchPlaceResponse fetchPlaceResponse) {
                                                                    Place place = fetchPlaceResponse.getPlace();
                                                                    LatLng placeLatLng = place.getLatLng();
                                                                    if (placeLatLng != null) {
                                                                        // Add marker into map for each found place
                                                                        map.addMarker(new MarkerOptions().position(placeLatLng).title(place.getName()));
                                                                        // Calculate distance between found place and current location
                                                                        double distance = calculateDistance(currentLocation, placeLatLng);
                                                                        Log.d(TAG,"Distance is: " + distance);
                                                                        // Save if it is nearest
                                                                        if (distance < minDistance[0]) {
                                                                            minDistance[0] = distance;
                                                                            nearestPlaceLatLng[0] = placeLatLng;
                                                                        }
                                                                    }
                                                                    // Increment processed places counter
                                                                    placesProcessed.incrementAndGet();
                                                                    // If all places has been processed
                                                                    if ((placesProcessed.get() == totalPlaces) && (nearestPlaceLatLng[0] != null)) {
                                                                        // Animate camera to go nearest found place
                                                                        Log.d(TAG,"Goes to: " + nearestPlaceLatLng[0]);
                                                                        map.animateCamera(CameraUpdateFactory.newLatLngZoom(nearestPlaceLatLng[0], 15f), 4000, null);
                                                                    }
                                                                }
                                                            })
                                                            .addOnFailureListener(new OnFailureListener() {
                                                                @Override
                                                                public void onFailure(@NonNull Exception e) {
                                                                    Log.e(TAG, "Place not found: " + e.getMessage());
                                                                    // Increment processed places counter
                                                                    placesProcessed.incrementAndGet();
                                                                    // If all places has been processed
                                                                    if ((placesProcessed.get() == totalPlaces) && (nearestPlaceLatLng[0] != null)) {
                                                                        // Animate camera to go nearest found place
                                                                        Log.d(TAG,"Goes to: " + nearestPlaceLatLng[0]);
                                                                        map.animateCamera(CameraUpdateFactory.newLatLngZoom(nearestPlaceLatLng[0], 15f), 4000, null);
                                                                    }
                                                                }
                                                            });
                                                }
                                            }
                                        })
                                        .addOnFailureListener(new OnFailureListener() {
                                            @Override
                                            public void onFailure(@NonNull Exception e) {
                                                Log.e(TAG, "Autocomplete request failed: " + e.getMessage());
                                            }
                                        });
                            }
                        }
                    });
        } else {
            Toast.makeText(getActivity(), getString(R.string.search_permission), Toast.LENGTH_SHORT).show();
        }
    }
    private double calculateDistance(LatLng start, LatLng end) {
        // Haversine method to calculate distance over spherical surface
        double earthRadius = 6371; // Kilometers
        double dLat = Math.toRadians(end.latitude - start.latitude);
        double dLng = Math.toRadians(end.longitude - start.longitude);
        double a = Math.sin(dLat / 2) * Math.sin(dLat / 2) + Math.cos(Math.toRadians(start.latitude)) * Math.cos(Math.toRadians(end.latitude)) * Math.sin(dLng / 2) * Math.sin(dLng / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        return earthRadius * c;
    }
}